<?php
/**
 * @file
 * Functions:
 *
 *  - Helpers to determine entity translatability
 *  - Callback for field widget
 */

/**
 * Determines if an entity is translatable.
 *
 * @param string $entity_type
 *   The type of entity.
 * @param object $entity
 *   The entity object.
 * @param string $bundle
 *   A bundle of the entity.
 *
 * @return bool
 *   Whether an entity is translatable.
 */
function translation_field_diffuser_entity_is_translatable($entity_type, $entity = NULL, $bundle = NULL) {
  $translatable = FALSE;

  if ($entity_type == 'node') {
    $type = is_object($entity) ? $entity->type : (!empty($bundle) ? $bundle : '');

    $translatable = module_exists('translation') && translation_supported_type($type);
    $translatable |= module_exists('i18n_node') && i18n_node_type_enabled($type);
  }

  if (module_exists('i18n_taxonomy') && $entity_type == 'taxonomy_term'
    && is_object($entity)) {
    // @todo currently not working, set to FALSE
    $translatable = FALSE;
  }

  if (module_exists('i18n_taxonomy') && $entity_type == 'vocabulary'
    && (is_object($entity) || $bundle)) {
    $entity = !empty($entity) ? $entity : taxonomy_vocabulary_machine_name_load($bundle);
    if (property_exists($entity, 'i18n_mode')) {
      $translatable |= $entity->i18n_mode == I18N_MODE_TRANSLATE;
    }
    else {
      $translatable |= i18n_taxonomy_vocabulary_mode($entity) == I18N_MODE_TRANSLATE;
    }
  }

  if (module_exists('entity_translation')) {
    $translatable |= entity_translation_enabled($entity_type, $entity);
  }

  return $translatable;
}

/**
 * Determines if at least one field for an entity is translatable.
 *
 * @param string $type
 *   The machine name entity type.
 *
 * @return bool
 *   Whether at least one field for an entity is translatable.
 */
function translation_field_diffuser_entity_field_is_translatable($type) {
  $translatable = &drupal_static(__FUNCTION__ . '_' . $type, FALSE);

  if (module_exists('entity_translation') && !$translatable) {
    foreach (entity_get_info() as $entity_type => $info) {
      if ($type == $entity_type && $info['fieldable']
        && entity_translation_enabled($entity_type)) {
        $translatable = TRUE;
      }
    }
  }

  return $translatable;
}

/**
 * Callback function to process the field.
 *
 * Add a list of fields that can propagate their data from
 * the entity source to its translations.
 *
 * @param array $element
 *   The field widget form element.
 */
function translation_field_diffuser_field_element_add_process(array &$element) {
  $field_name = $element['#field_name'];
  $name = 'translation_field_diffuser_' . $element['#entity_type'] . '_' . $element['#bundle'];
  $name .= '_' . $field_name;

  if (variable_get($name . '_value', FALSE)) {
    if (!isset($element['#process'])) {
      $element['#process'] = array();
      // Get process function(s) for custom form element type.
      if (isset($element['#type'])) {
        $info = element_info($element['#type']);
        $element['#process'] = isset($info['#process']) ? $info['#process'] : array();
      }
    }

    $element['#process'][] = 'translation_field_diffuser_widget_process';

    if (property_exists($element['#entity'], 'propagable_fields')) {
      // Avoid duplicate with multi value element form
      if (!in_array($field_name, $element['#entity']->propagable_fields)) {
        $element['#entity']->propagable_fields[] = $field_name;
      }
    }
    else {
      $element['#entity']->propagable_fields = array($field_name);
    }
  }
}